# !/usr/bin/env python
# -*-coding:utf-8 -*-
# Author     ：Campanula 梦芸 何
import sqlite3
from typing import Iterable, Dict


from anduin.common import ENGINE_SQLITE,dbg,get_obj_name
from anduin.parser.sql_parser import Parser
from anduin.frames.client_base import ClientBase


class SQLiteClient(ClientBase):
    def __init__(self, *args):
        super().__init__(*args)
        self.db = self.connect_db()
        self._tables: Dict[str, Iterable] = {}

    def connect_db(self):
        try:
            res  =  sqlite3.connect(self._dbname)
            dbg('连接创建成功', get_obj_name(self))
            return res
        except Exception as e:
            dbg('连接创建失败', e,get_obj_name(self))
            return


    def load_an_table(self, tablename):
        sql = 'PRAGMA table_info(%s)' % tablename
        res = self.query(sql, show_sql=False)
        if res is None:
            return
        if isinstance(res, Iterable):
            column_list = list(map(lambda x: x[0], res))
            self._tables[tablename] = column_list
            return column_list

    def query(self, sql, show_sql=False, sql_params=None):
        dummy_sql = sql % tuple(sql_params) if sql_params is not None else sql
        sql = sql.replace('binary', '')
        sql = sql.replace('%s', '?')
        if show_sql is True:
            dbg('sql_id', id(self), dummy_sql)
        cursor = self.db.cursor()

        try:
            if sql_params is not None:
                cursor.execute(sql, tuple(sql_params))
            else:
                cursor.execute(sql)
            results = cursor.fetchall()
        except Exception as e:
            dbg('<--------DBERROR-------->')
            dbg(dummy_sql)
            dbg('execute fail!', str(e))
            dbg('<--------DBERROR-------->')
            results = e
        return results

    def create(self, table, columns, table_comment='', show_sql=False):
        sql = Parser.create_table_parser(table, columns, table_comment, sql_engine=ENGINE_SQLITE)
        self.query(sql, show_sql)
        self.commit()
        return

    # 查找数据（单条）
    def find(self, table, conditions, or_cond=None, fields=('*',), order=None, show_sql=False, for_update=False):
        if table not in self._tables:
            self.load_an_table(table)

        if fields[0] == '*' and len(fields) == 1:
            fieldList = self._tables[table]
            fields = fieldList

        sql, sql_params = Parser.find_info(table, conditions, or_cond, fields, None, order, None, for_update)
        if sql is None:
            return

        sql += " limit 1"
        res = self.query(sql, show_sql, sql_params)
        if isinstance(res, Exception) is True:
            return res
        if 0 == len(res):
            return None
        result = dict(zip(fields, res[0]))
        return result

    # 查找数据
    def select(self, table, conditions, or_cond=None, fields=('*',), group=None, order=None, limit=None, show_sql=False,
               for_update=False):
        if table not in self._tables:
            self.load_an_table(table)

        sql, sql_params = Parser.find_info(table, conditions, or_cond, fields, group, order, limit, for_update)
        if sql is None:
            return
        #
        res = self.query(sql, show_sql, sql_params)
        if isinstance(res, Exception) is True:
            return res
        if 0 == len(res):
            return None
        result = []
        for data in res:
            data = dict(zip(fields, data))
            result.append(data)

        return result

    def insert(self, table, content, show_sql=False):
        sql, sql_params = Parser.insert_parser(table, content)
        r = self.query(sql, show_sql, sql_params)
        return r

    def update(self, table, conditions, or_cond=None, params=None, show_sql=False):
        # dbg('开始执行')
        sql, sql_params = Parser.update_parser(table, conditions, or_cond, params)
        r = self.query(sql, show_sql, sql_params)
        # dbg('自动提交完毕')
        return r

    def delete(self, table, condition, or_cond=None, show_sql=False):
        sql = 'delete from %s where  ' % table
        sql, sql_params = Parser.bind_conditions(sql, condition, or_cond)
        #  #
        r = self.query(sql, show_sql, sql_params)
        return r



import datetime
import functools
import inspect
import os
import sqlite3
import sys
import time
from functools import wraps
from typing import Any

import aredis
import pymysql

ENGINE_SQLITE = 'sqlite'
ENGINE_MYSQL = 'mysql'
ENGINE_REDIS = 'async_redis'
danger_sig = ['--', '-+', '#']
start_time = int(time.time())
ENGINE_DICT = {
    ENGINE_MYSQL: pymysql,
    ENGINE_SQLITE: sqlite3,
    ENGINE_REDIS:aredis,
}
sql_clean_time = 45
LOG_PATH = '%s/.anduin/' % os.path.expanduser('~')


def time_to_str(times):
    date_array = datetime.datetime.utcfromtimestamp(times + (8 * 3600))
    return date_array.strftime("%Y-%m-%d %H:%M:%S")


def get_filename():
    if sys.platform != 'win32':
        fn = '%s%s-%s.log' % (LOG_PATH, sys.argv[0].split('/')[-1], start_time)
    else:
        fn = '%s\\.anduin\\%s-%s.log' % (os.path.expanduser('~'), sys.argv[0].split('\\')[-1], start_time)
    return fn

fn = get_filename()
if sys.platform != 'win32':
    try:
        os.mkdir('%s/.anduin' % os.path.expanduser('~'))
    except Exception as e:
        print(str(e))
    print('系统内核为',sys.platform,'本插件调用日志保存在%s' % get_filename())
else:
    print('windows下不支持持久化日志，请自行控制台生产')
fh = open(fn, 'a')


if sys.platform != 'win32':
    write_log = True
else:
    write_log = False


def dbg(*args):
    res = ['[%s Anduin Engine]' % time_to_str(int(time.time()))] + list(args)
    print(*res)
    if write_log:
        for i in res:
            fh.write(str(i) + ' ')
        fh.write('\n')
        # fh.close()


def get_db_index(db_config_dict):
    return '%s@%s:%s'%(db_config_dict.get('user'),db_config_dict.get('host'),db_config_dict.get('database'))


def func_time(f):
    """
    简单记录执行时间
    :param f:
    :return:
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = f(*args, **kwargs)
        end = time.time()
        info = str(f.__name__) + ' took ' + str(end - start) + ' seconds '
        dbg(info)
        return result

    return wrapper


def can_return_directly(res):
    if res is None:
        return True
    if res is False:
        return True
    if isinstance(res, Exception):
        return True


def clean_old_log(filename='*.log'):
    if sys.platform != 'win32':
        os.system('rm %s%s' % (LOG_PATH, filename))


def async_decorators(method):
    """
    异步装饰器装饰异步方法
    :param method: 被装饰协程（异步方法）
    :return:
    """

    @functools.wraps(method)
    async def wrapper(*args, **kwargs):
        # 异步执行
        # 此处必须await 切换调用被装饰协程，否则不会运行该协程
        data = method(*args, **kwargs)
        return data

    return wrapper

def get_obj_name(obj: Any) -> str:
    """获取对象的名称"""
    if inspect.isfunction(obj) or inspect.isclass(obj):
        return obj.__name__
    return obj.__class__.__name__


"""Anduin: A light python mysql connector.

Copyright (c) 2020-2024 Campanula<campanulamediuml@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE."""
# <=========>

from .common.tools import get_filename, clean_old_log, dbg, get_obj_name
from .db.no_sql.async_redis.redis_client_manager import AsyncRedisManager
from .db.sql.async_mysql.db_client_manager import AsyncMySQLManager
from .db.sql.mysql.db_client_manager import MySQLManager
from .db.sql.sqlite.db_client_manager import SQLiteManager

AMySQL = AsyncMySQLManager
MySQL = MySQLManager
SQLite = SQLiteManager
ARedis = AsyncRedisManager


conf_map = {

}
def auto_init():
    dbg('自动初始化同步mysql')
    try:
        from config.db_config import db_config
        mysql_pool = MySQL(db_config)
        dbg('自动初始化同步mysql成功')
        return mysql_pool
    except Exception as e:
        dbg(e)

Data = auto_init()
if Data is None:
    dbg('自动初始化同步mysql失败')



__version__ = "8.3.0"



